# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/event_service.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import uri_pb2 as farm__ng_dot_core_dot_uri__pb2
from farm_ng.core import event_pb2 as farm__ng_dot_core_dot_event__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n farm_ng/core/event_service.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x16\x66\x61rm_ng/core/uri.proto\x1a\x18\x66\x61rm_ng/core/event.proto\"I\n\x10SubscribeRequest\x12$\n\x03uri\x18\x01 \x01(\x0b\x32\x17.farm_ng.core.proto.Uri\x12\x0f\n\x07\x65very_n\x18\x02 \x01(\r\"K\n\x0eSubscribeReply\x12(\n\x05\x65vent\x18\x01 \x01(\x0b\x32\x19.farm_ng.core.proto.Event\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"P\n\x13RequestReplyRequest\x12(\n\x05\x65vent\x18\x01 \x01(\x0b\x32\x19.farm_ng.core.proto.Event\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"N\n\x11RequestReplyReply\x12(\n\x05\x65vent\x18\x01 \x01(\x0b\x32\x19.farm_ng.core.proto.Event\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"~\n\x0cRequestReply\x12\x38\n\x07request\x18\x01 \x01(\x0b\x32\'.farm_ng.core.proto.RequestReplyRequest\x12\x34\n\x05reply\x18\x02 \x01(\x0b\x32%.farm_ng.core.proto.RequestReplyReply\"\x11\n\x0fListUrisRequest\"6\n\rListUrisReply\x12%\n\x04uris\x18\x01 \x03(\x0b\x32\x17.farm_ng.core.proto.Uri\"\xe0\x02\n\x12\x45ventServiceConfig\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x0c\n\x04host\x18\x03 \x01(\t\x12;\n\rsubscriptions\x18\x04 \x03(\x0b\x32$.farm_ng.core.proto.SubscribeRequest\x12%\n\x04uris\x18\x05 \x03(\x0b\x32\x17.farm_ng.core.proto.Uri\x12\x0c\n\x04\x61rgs\x18\x06 \x03(\t\x12\x42\n\tlog_level\x18\x07 \x01(\x0e\x32/.farm_ng.core.proto.EventServiceConfig.LogLevel\"j\n\x08LogLevel\x12\n\n\x06NOTSET\x10\x00\x12\x0c\n\x08\x43RITICAL\x10\x32\x12\t\n\x05\x46\x41TAL\x10\x32\x12\t\n\x05\x45RROR\x10(\x12\x0b\n\x07WARNING\x10\x1e\x12\x08\n\x04WARN\x10\x1e\x12\x08\n\x04INFO\x10\x14\x12\t\n\x05\x44\x45\x42UG\x10\n\x1a\x02\x10\x01\"Q\n\x16\x45ventServiceConfigList\x12\x37\n\x07\x63onfigs\x18\x01 \x03(\x0b\x32&.farm_ng.core.proto.EventServiceConfig2\xa1\x02\n\x0c\x45ventService\x12T\n\x08listUris\x12#.farm_ng.core.proto.ListUrisRequest\x1a!.farm_ng.core.proto.ListUrisReply\"\x00\x12Y\n\tsubscribe\x12$.farm_ng.core.proto.SubscribeRequest\x1a\".farm_ng.core.proto.SubscribeReply\"\x00\x30\x01\x12`\n\x0crequestReply\x12\'.farm_ng.core.proto.RequestReplyRequest\x1a%.farm_ng.core.proto.RequestReplyReply\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.event_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENTSERVICECONFIG_LOGLEVEL']._loaded_options = None
  _globals['_EVENTSERVICECONFIG_LOGLEVEL']._serialized_options = b'\020\001'
  _globals['_SUBSCRIBEREQUEST']._serialized_start=106
  _globals['_SUBSCRIBEREQUEST']._serialized_end=179
  _globals['_SUBSCRIBEREPLY']._serialized_start=181
  _globals['_SUBSCRIBEREPLY']._serialized_end=256
  _globals['_REQUESTREPLYREQUEST']._serialized_start=258
  _globals['_REQUESTREPLYREQUEST']._serialized_end=338
  _globals['_REQUESTREPLYREPLY']._serialized_start=340
  _globals['_REQUESTREPLYREPLY']._serialized_end=418
  _globals['_REQUESTREPLY']._serialized_start=420
  _globals['_REQUESTREPLY']._serialized_end=546
  _globals['_LISTURISREQUEST']._serialized_start=548
  _globals['_LISTURISREQUEST']._serialized_end=565
  _globals['_LISTURISREPLY']._serialized_start=567
  _globals['_LISTURISREPLY']._serialized_end=621
  _globals['_EVENTSERVICECONFIG']._serialized_start=624
  _globals['_EVENTSERVICECONFIG']._serialized_end=976
  _globals['_EVENTSERVICECONFIG_LOGLEVEL']._serialized_start=870
  _globals['_EVENTSERVICECONFIG_LOGLEVEL']._serialized_end=976
  _globals['_EVENTSERVICECONFIGLIST']._serialized_start=978
  _globals['_EVENTSERVICECONFIGLIST']._serialized_end=1059
  _globals['_EVENTSERVICE']._serialized_start=1062
  _globals['_EVENTSERVICE']._serialized_end=1351
# @@protoc_insertion_point(module_scope)
