# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/pose.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import lie_pb2 as farm__ng_dot_core_dot_lie__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17\x66\x61rm_ng/core/pose.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x16\x66\x61rm_ng/core/lie.proto\"\xa0\x01\n\x04Pose\x12\x32\n\x08\x61_from_b\x18\x01 \x01(\x0b\x32 .farm_ng.core.proto.Isometry3F64\x12\x0f\n\x07\x66rame_a\x18\x02 \x01(\t\x12\x0f\n\x07\x66rame_b\x18\x03 \x01(\t\x12\x42\n\x11tangent_of_b_in_a\x18\x04 \x01(\x0b\x32\'.farm_ng.core.proto.Isometry3F64Tangent\"3\n\x08PoseTree\x12\'\n\x05poses\x18\x01 \x03(\x0b\x32\x18.farm_ng.core.proto.Poseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.pose_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_POSE']._serialized_start=72
  _globals['_POSE']._serialized_end=232
  _globals['_POSETREE']._serialized_start=234
  _globals['_POSETREE']._serialized_end=285
# @@protoc_insertion_point(module_scope)
