# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/image.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x66\x61rm_ng/core/image.proto\x12\x12\x66\x61rm_ng.core.proto\"*\n\tImageSize\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\"O\n\x0bImageLayout\x12+\n\x04size\x18\x01 \x01(\x0b\x32\x1d.farm_ng.core.proto.ImageSize\x12\x13\n\x0bpitch_bytes\x18\x02 \x01(\r\"[\n\x0bPixelFormat\x12\x13\n\x0bnumber_type\x18\x01 \x01(\t\x12\x16\n\x0enum_components\x18\x02 \x01(\x05\x12\x1f\n\x17num_bytes_per_component\x18\x03 \x01(\x05\"\x80\x01\n\x08\x44ynImage\x12/\n\x06layout\x18\x01 \x01(\x0b\x32\x1f.farm_ng.core.proto.ImageLayout\x12\x35\n\x0cpixel_format\x18\x02 \x01(\x0b\x32\x1f.farm_ng.core.proto.PixelFormat\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.image_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_IMAGESIZE']._serialized_start=48
  _globals['_IMAGESIZE']._serialized_end=90
  _globals['_IMAGELAYOUT']._serialized_start=92
  _globals['_IMAGELAYOUT']._serialized_end=171
  _globals['_PIXELFORMAT']._serialized_start=173
  _globals['_PIXELFORMAT']._serialized_end=264
  _globals['_DYNIMAGE']._serialized_start=267
  _globals['_DYNIMAGE']._serialized_end=395
# @@protoc_insertion_point(module_scope)
