# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/recorder.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import event_service_pb2 as farm__ng_dot_core_dot_event__service__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x61rm_ng/core/recorder.proto\x12\x12\x66\x61rm_ng.core.proto\x1a farm_ng/core/event_service.proto\"\\\n\x0fRecorderProfile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12;\n\rsubscriptions\x18\x02 \x03(\x0b\x32$.farm_ng.core.proto.SubscribeRequestb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.recorder_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_RECORDERPROFILE']._serialized_start=85
  _globals['_RECORDERPROFILE']._serialized_end=177
# @@protoc_insertion_point(module_scope)
