# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/std.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x66\x61rm_ng/core/std.proto\x12\x12\x66\x61rm_ng.core.proto\"3\n\x0fOptionalG0Float\x12\r\n\x05value\x18\x01 \x01(\x02\x12\x11\n\thas_value\x18\x02 \x01(\x08\"4\n\x10OptionalG0Double\x12\r\n\x05value\x18\x01 \x01(\x01\x12\x11\n\thas_value\x18\x02 \x01(\x08\"3\n\x0fOptionalG0Int32\x12\r\n\x05value\x18\x01 \x01(\x05\x12\x11\n\thas_value\x18\x02 \x01(\x08\"3\n\x0fOptionalG0Int64\x12\r\n\x05value\x18\x01 \x01(\x03\x12\x11\n\thas_value\x18\x02 \x01(\x08\"2\n\x0eOptionalG0Bool\x12\r\n\x05value\x18\x01 \x01(\x08\x12\x11\n\thas_value\x18\x02 \x01(\x08\"4\n\x10OptionalG0String\x12\r\n\x05value\x18\x01 \x01(\t\x12\x11\n\thas_value\x18\x02 \x01(\x08\"%\n\x0e\x46ileSystemPath\x12\x13\n\x0bpath_string\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.std_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_OPTIONALG0FLOAT']._serialized_start=46
  _globals['_OPTIONALG0FLOAT']._serialized_end=97
  _globals['_OPTIONALG0DOUBLE']._serialized_start=99
  _globals['_OPTIONALG0DOUBLE']._serialized_end=151
  _globals['_OPTIONALG0INT32']._serialized_start=153
  _globals['_OPTIONALG0INT32']._serialized_end=204
  _globals['_OPTIONALG0INT64']._serialized_start=206
  _globals['_OPTIONALG0INT64']._serialized_end=257
  _globals['_OPTIONALG0BOOL']._serialized_start=259
  _globals['_OPTIONALG0BOOL']._serialized_end=309
  _globals['_OPTIONALG0STRING']._serialized_start=311
  _globals['_OPTIONALG0STRING']._serialized_end=363
  _globals['_FILESYSTEMPATH']._serialized_start=365
  _globals['_FILESYSTEMPATH']._serialized_end=402
# @@protoc_insertion_point(module_scope)
