# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/geometry.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import linalg_pb2 as farm__ng_dot_core_dot_linalg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x61rm_ng/core/geometry.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x19\x66\x61rm_ng/core/linalg.proto\"8\n\x0bUnitVec3F64\x12)\n\x04vec3\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\"Q\n\x0eHyperplane3F64\x12/\n\x06normal\x18\x01 \x01(\x0b\x32\x1f.farm_ng.core.proto.UnitVec3F64\x12\x0e\n\x06offset\x18\x02 \x01(\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.geometry_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_UNITVEC3F64']._serialized_start=78
  _globals['_UNITVEC3F64']._serialized_end=134
  _globals['_HYPERPLANE3F64']._serialized_start=136
  _globals['_HYPERPLANE3F64']._serialized_end=217
# @@protoc_insertion_point(module_scope)
