# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/lie.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import linalg_pb2 as farm__ng_dot_core_dot_linalg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x66\x61rm_ng/core/lie.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x19\x66\x61rm_ng/core/linalg.proto\"H\n\rQuaternionF64\x12)\n\x04imag\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\x12\x0c\n\x04real\x18\x02 \x01(\x01\"\x1d\n\x0cRotation2F64\x12\r\n\x05theta\x18\x01 \x01(\x01\"t\n\x0cIsometry2F64\x12\x32\n\x08rotation\x18\x01 \x01(\x0b\x32 .farm_ng.core.proto.Rotation2F64\x12\x30\n\x0btranslation\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F64\"J\n\x0cRotation3F64\x12:\n\x0funit_quaternion\x18\x01 \x01(\x0b\x32!.farm_ng.core.proto.QuaternionF64\"t\n\x0cIsometry3F64\x12\x32\n\x08rotation\x18\x01 \x01(\x0b\x32 .farm_ng.core.proto.Rotation3F64\x12\x30\n\x0btranslation\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\"\x82\x01\n\x13Isometry3F64Tangent\x12\x34\n\x0flinear_velocity\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\x12\x35\n\x10\x61ngular_velocity\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.lie_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_QUATERNIONF64']._serialized_start=73
  _globals['_QUATERNIONF64']._serialized_end=145
  _globals['_ROTATION2F64']._serialized_start=147
  _globals['_ROTATION2F64']._serialized_end=176
  _globals['_ISOMETRY2F64']._serialized_start=178
  _globals['_ISOMETRY2F64']._serialized_end=294
  _globals['_ROTATION3F64']._serialized_start=296
  _globals['_ROTATION3F64']._serialized_end=370
  _globals['_ISOMETRY3F64']._serialized_start=372
  _globals['_ISOMETRY3F64']._serialized_end=488
  _globals['_ISOMETRY3F64TANGENT']._serialized_start=491
  _globals['_ISOMETRY3F64TANGENT']._serialized_end=621
# @@protoc_insertion_point(module_scope)
