# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/sensor.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import image_pb2 as farm__ng_dot_core_dot_image__pb2
from farm_ng.core import lie_pb2 as farm__ng_dot_core_dot_lie__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x66\x61rm_ng/core/sensor.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x18\x66\x61rm_ng/core/image.proto\x1a\x16\x66\x61rm_ng/core/lie.proto\"i\n\x0b\x43\x61meraModel\x12\x31\n\nimage_size\x18\x01 \x01(\x0b\x32\x1d.farm_ng.core.proto.ImageSize\x12\x17\n\x0f\x64istortion_type\x18\x02 \x01(\t\x12\x0e\n\x06params\x18\x03 \x03(\x01\"F\n\x0c\x43\x61meraModels\x12\x36\n\rcamera_models\x18\x01 \x03(\x0b\x32\x1f.farm_ng.core.proto.CameraModel\"+\n\x0e\x43lippingPlanes\x12\x0c\n\x04near\x18\x01 \x01(\x02\x12\x0b\n\x03\x66\x61r\x18\x02 \x01(\x02\"}\n\x0bRigidCamera\x12\x33\n\nintrinsics\x18\x01 \x01(\x0b\x32\x1f.farm_ng.core.proto.CameraModel\x12\x39\n\x0frig_from_camera\x18\x02 \x01(\x0b\x32 .farm_ng.core.proto.Isometry3F64\"/\n\tGyroModel\x12\x12\n\nmodel_type\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x03(\x01\"3\n\rAcceleroModel\x12\x12\n\nmodel_type\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x03(\x01\"x\n\x08ImuModel\x12\x31\n\ngyro_model\x18\x01 \x01(\x0b\x32\x1d.farm_ng.core.proto.GyroModel\x12\x39\n\x0e\x61\x63\x63\x65lero_model\x18\x02 \x01(\x0b\x32!.farm_ng.core.proto.AcceleroModelb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.sensor_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CAMERAMODEL']._serialized_start=99
  _globals['_CAMERAMODEL']._serialized_end=204
  _globals['_CAMERAMODELS']._serialized_start=206
  _globals['_CAMERAMODELS']._serialized_end=276
  _globals['_CLIPPINGPLANES']._serialized_start=278
  _globals['_CLIPPINGPLANES']._serialized_end=321
  _globals['_RIGIDCAMERA']._serialized_start=323
  _globals['_RIGIDCAMERA']._serialized_end=448
  _globals['_GYROMODEL']._serialized_start=450
  _globals['_GYROMODEL']._serialized_end=497
  _globals['_ACCELEROMODEL']._serialized_start=499
  _globals['_ACCELEROMODEL']._serialized_end=550
  _globals['_IMUMODEL']._serialized_start=552
  _globals['_IMUMODEL']._serialized_end=672
# @@protoc_insertion_point(module_scope)
