# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/calculus.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import linalg_pb2 as farm__ng_dot_core_dot_linalg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x61rm_ng/core/calculus.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x19\x66\x61rm_ng/core/linalg.proto\"7\n\tRegionI32\x12\x10\n\x08is_empty\x18\x01 \x01(\x08\x12\x0b\n\x03min\x18\x02 \x01(\x05\x12\x0b\n\x03max\x18\x03 \x01(\x05\"7\n\tRegionF32\x12\x10\n\x08is_empty\x18\x01 \x01(\x08\x12\x0b\n\x03min\x18\x02 \x01(\x02\x12\x0b\n\x03max\x18\x03 \x01(\x02\"7\n\tRegionF64\x12\x10\n\x08is_empty\x18\x01 \x01(\x08\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0b\n\x03max\x18\x03 \x01(\x01\"r\n\nRegion2F64\x12\x10\n\x08is_empty\x18\x01 \x01(\x08\x12(\n\x03min\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F64\x12(\n\x03max\x18\x03 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F64\"E\n\x14RepeatedG0Region2F64\x12-\n\x05value\x18\x01 \x03(\x0b\x32\x1e.farm_ng.core.proto.Region2F64b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.calculus_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_REGIONI32']._serialized_start=78
  _globals['_REGIONI32']._serialized_end=133
  _globals['_REGIONF32']._serialized_start=135
  _globals['_REGIONF32']._serialized_end=190
  _globals['_REGIONF64']._serialized_start=192
  _globals['_REGIONF64']._serialized_end=247
  _globals['_REGION2F64']._serialized_start=249
  _globals['_REGION2F64']._serialized_end=363
  _globals['_REPEATEDG0REGION2F64']._serialized_start=365
  _globals['_REPEATEDG0REGION2F64']._serialized_end=434
# @@protoc_insertion_point(module_scope)
