# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/linalg.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x66\x61rm_ng/core/linalg.proto\x12\x12\x66\x61rm_ng.core.proto\"\x1f\n\x07Vec2I64\x12\t\n\x01x\x18\x01 \x01(\x03\x12\t\n\x01y\x18\x02 \x01(\x03\"\x1f\n\x07Vec2F32\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\"\x1f\n\x07Vec2F64\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\"*\n\x07Vec3I64\x12\t\n\x01x\x18\x01 \x01(\x03\x12\t\n\x01y\x18\x02 \x01(\x03\x12\t\n\x01z\x18\x03 \x01(\x03\"*\n\x07Vec3F32\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"*\n\x07Vec3F64\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\"a\n\x07Mat2F32\x12*\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F32\x12*\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F32\"a\n\x07Mat2F64\x12*\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F64\x12*\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec2F64\"\x8d\x01\n\x07Mat3F32\x12*\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F32\x12*\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F32\x12*\n\x05\x63ol_2\x18\x03 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F32\"\x8d\x01\n\x07Mat3F64\x12*\n\x05\x63ol_0\x18\x01 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\x12*\n\x05\x63ol_1\x18\x02 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\x12*\n\x05\x63ol_2\x18\x03 \x01(\x0b\x32\x1b.farm_ng.core.proto.Vec3F64\"\x17\n\x07VecXF32\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x02\"\x17\n\x07VecXF64\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\x01\"3\n\x07MatXF32\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x02\"3\n\x07MatXF64\x12\x0c\n\x04rows\x18\x01 \x01(\x05\x12\x0c\n\x04\x63ols\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x03(\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.linalg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_VEC2I64']._serialized_start=49
  _globals['_VEC2I64']._serialized_end=80
  _globals['_VEC2F32']._serialized_start=82
  _globals['_VEC2F32']._serialized_end=113
  _globals['_VEC2F64']._serialized_start=115
  _globals['_VEC2F64']._serialized_end=146
  _globals['_VEC3I64']._serialized_start=148
  _globals['_VEC3I64']._serialized_end=190
  _globals['_VEC3F32']._serialized_start=192
  _globals['_VEC3F32']._serialized_end=234
  _globals['_VEC3F64']._serialized_start=236
  _globals['_VEC3F64']._serialized_end=278
  _globals['_MAT2F32']._serialized_start=280
  _globals['_MAT2F32']._serialized_end=377
  _globals['_MAT2F64']._serialized_start=379
  _globals['_MAT2F64']._serialized_end=476
  _globals['_MAT3F32']._serialized_start=479
  _globals['_MAT3F32']._serialized_end=620
  _globals['_MAT3F64']._serialized_start=623
  _globals['_MAT3F64']._serialized_end=764
  _globals['_VECXF32']._serialized_start=766
  _globals['_VECXF32']._serialized_end=789
  _globals['_VECXF64']._serialized_start=791
  _globals['_VECXF64']._serialized_end=814
  _globals['_MATXF32']._serialized_start=816
  _globals['_MATXF32']._serialized_end=867
  _globals['_MATXF64']._serialized_start=869
  _globals['_MATXF64']._serialized_end=920
# @@protoc_insertion_point(module_scope)
