# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/event.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from farm_ng.core import uri_pb2 as farm__ng_dot_core_dot_uri__pb2
from farm_ng.core import timestamp_pb2 as farm__ng_dot_core_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x66\x61rm_ng/core/event.proto\x12\x12\x66\x61rm_ng.core.proto\x1a\x16\x66\x61rm_ng/core/uri.proto\x1a\x1c\x66\x61rm_ng/core/timestamp.proto\"\x8a\x01\n\x05\x45vent\x12$\n\x03uri\x18\x01 \x01(\x0b\x32\x17.farm_ng.core.proto.Uri\x12\x31\n\ntimestamps\x18\x02 \x03(\x0b\x32\x1d.farm_ng.core.proto.Timestamp\x12\x16\n\x0epayload_length\x18\x03 \x01(\x03\x12\x10\n\x08sequence\x18\x04 \x01(\x03\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.event_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EVENT']._serialized_start=103
  _globals['_EVENT']._serialized_end=241
# @@protoc_insertion_point(module_scope)
