from farm_ng.core import event_service_pb2 as _event_service_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class RecorderProfile(_message.Message):
    __slots__ = ("name", "subscriptions")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SUBSCRIPTIONS_FIELD_NUMBER: _ClassVar[int]
    name: str
    subscriptions: _containers.RepeatedCompositeFieldContainer[_event_service_pb2.SubscribeRequest]
    def __init__(self, name: _Optional[str] = ..., subscriptions: _Optional[_Iterable[_Union[_event_service_pb2.SubscribeRequest, _Mapping]]] = ...) -> None: ...
