# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: farm_ng/core/examples.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x61rm_ng/core/examples.proto\x12\x12\x66\x61rm_ng.core.proto\",\n\x0eStructExample1\x12\x0f\n\x07integer\x18\x01 \x01(\x01\x12\t\n\x01\x66\x18\x02 \x01(\x02\"O\n\x0eStructExample2\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\x03\x65x1\x18\x02 \x01(\x0b\x32\".farm_ng.core.proto.StructExample1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'farm_ng.core.examples_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_STRUCTEXAMPLE1']._serialized_start=51
  _globals['_STRUCTEXAMPLE1']._serialized_end=95
  _globals['_STRUCTEXAMPLE2']._serialized_start=97
  _globals['_STRUCTEXAMPLE2']._serialized_end=176
# @@protoc_insertion_point(module_scope)
